<?php

namespace Drupal\commerce_wechat_pay\PluginForm;

use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm;
use Drupal\Core\Form\FormStateInterface;
use EasyWeChat\Foundation\Application;
use EasyWeChat\Payment\Order;
use GuzzleHttp\Exception\GuzzleException;
use Drupal\commerce_price\Price;

class MicropayForm extends PaymentOffsiteForm {

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['barcode'] = [
      '#type' => 'number',
      '#title' => t('Barcode'),
      '#size' => 18
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValue($form['#parents']);

    // 18 digtis, must start with 11, 12, 13, 14 or 15
    if (preg_match('/^1(0|1|2|3|4|5)\d{16}$/', $values['barcode']) == 0) {
      $form_state->setError($form['barcode'], t('Invalid barcode detected! The barcode should have 18 digits and it must start with 11, 12, 13, 14 or 15. The invalid barcode is: ') . $values['barcode']);
      return;
    }

    /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
    $payment = $this->entity;
    /** @var \Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayInterface $payment_gateway_plugin */
    $payment_gateway_plugin = $payment->getPaymentGateway()->getPlugin();
    $appid = $payment_gateway_plugin->getConfiguration()['appid'];
    $mch_id = $payment_gateway_plugin->getConfiguration()['mch_id'];
    $key = $payment_gateway_plugin->getConfiguration()['key'];
    $sub_appid = $payment_gateway_plugin->getConfiguration()['appid'];
    $sub_mch_id = $payment_gateway_plugin->getConfiguration()['mch_id'];
    $order = $payment->getOrder();

    $options = [
      // 前面的appid什么的也得保留哦
      'app_id' => $appid,
      // ...
      // payment
      'payment' => [
        'merchant_id'        => $mch_id,
        'key'                => $key,
        // 'device_info'     => '013467007045764',
        // 'sub_app_id'      => '',
        // 'sub_merchant_id' => '',
        // ...
      ],
    ];
    $app = new Application($options);
    $wechat_pay = $app->payment;

    $attributes = [
      'auth_code' => $values['barcode'],
      'body'             => \Drupal::config('system.site')->get('name') . t(' Order: ') . $order->id(),
      // 'detail'           => 'order_items in certain format',
      'out_trade_no'     => $order->id(),
      'total_fee' => $payment->getAmount()->getNumber() * 100, // WeChat Pay use Integer for its price
      'fee_type' => $payment->getAmount()->getCurrencyCode(),
    ];
    $wechat_order = new Order($attributes);

    try {
      $result = $wechat_pay->pay($wechat_order);

      if ($result->return_code == 'SUCCESS') {

        if ($result->result_code == 'SUCCESS') {
          $form_state->setTemporary($result->toArray());
        } else if ($result->err_code == 'USERPAYING') {
          /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
          $payment = $this->entity;
          /** @var \Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayInterface $payment_gateway_plugin */
          $payment_gateway_plugin = $payment->getPaymentGateway()->getPlugin();
          $payment_gateway_plugin->createPayment($result->toArray(), 'capture', $payment->getOrder(), 'USERPAYING');

          $form_state->setError($form['barcode'], t('Customer is paying, please wait and check again. Message from WeChat Pay: ') . $result->err_code_des);
          return;

        } else if ($result->err_code == 'ORDERPAID') {

        } else {
          $form_state->setError($form['barcode'], t('Other error happened. Message from WeChat Pay: ') . $result->err_code_des);
          return;
        }

      } else { // For any reason, we cannot get a preorder made by WeChat service
        $form_state->setError($form['barcode'], t('Commerce WeChat Pay cannot process your payment: ') . $result->return_msg);
        return;
      }
    } catch (GuzzleException $e) {
      $form_state->setError($form['barcode'], t('Commerce WeChat Pay is having problem to connect to WeChat Pay servers: ') . $e->getMessage());
      return;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $result = $form_state->getTemporary();
    if ($result['result_code'] == 'SUCCESS') {
      /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
      $payment = $this->entity;
      /** @var \Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayInterface $payment_gateway_plugin */
      $payment_gateway_plugin = $payment->getPaymentGateway()->getPlugin();
      $payment_gateway_plugin->createPayment($result);
    } else if ($result['err_code'] == 'ORDERPAID') {
      // TODO Update payment information;
    }
  }

}
