<?php

namespace Drupal\commerce_wechat_pay\PluginForm;

use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm as BasePaymentOffsiteForm;
use Drupal\Core\Form\FormStateInterface;
use EasyWeChat\Foundation\Application;
use EasyWeChat\Payment\Order;
use Com\Tecnick\Barcode\Barcode;
use Drupal\Core\Render\Markup;

class QRCodePaymentForm extends BasePaymentOffsiteForm {

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
    $payment = $this->entity;
    /** @var \Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayInterface $payment_gateway_plugin */
    $payment_gateway_plugin = $payment->getPaymentGateway()->getPlugin();
    $appid = $payment_gateway_plugin->getConfiguration()['appid'];
    $mch_id = $payment_gateway_plugin->getConfiguration()['mch_id'];
    $key = $payment_gateway_plugin->getConfiguration()['key'];
    $sub_appid = $payment_gateway_plugin->getConfiguration()['appid'];
    $sub_mch_id = $payment_gateway_plugin->getConfiguration()['mch_id'];
    $order = $payment->getOrder();

    $options = [
      // 前面的appid什么的也得保留哦
      'app_id' => $appid,
      // ...
      // payment
      'payment' => [
        'merchant_id'        => $mch_id,
        'key'                => $key,
        //'cert_path'          => 'path/to/your/cert.pem', // XXX: 绝对路径！！！！
        //'key_path'           => 'path/to/your/key',      // XXX: 绝对路径！！！！
        'notify_url'         => $payment_gateway_plugin->getNotifyUrl()->toString(),
        // 'device_info'     => '013467007045764',
        // 'sub_app_id'      => '',
        // 'sub_merchant_id' => '',
        // ...
      ],
    ];
    $app = new Application($options);
    $wechat_pay = $app->payment;

    $attributes = [
      'trade_type'       => 'NATIVE', // JSAPI，NATIVE，APP...
      'body'             => \Drupal::config('system.site')->get('name') . t(' Order: ') . $order->id(),
      // 'detail'           => 'order_items in certain format',
      'out_trade_no'     => $order->id(),
      'total_fee' => $payment->getAmount()->getNumber() * 100, // WeChat Pay use Integer for its price
      'fee_type' => $payment->getAmount()->getCurrencyCode(),
    ];
    $wechat_order = new Order($attributes);

    $result = $wechat_pay->prepare($wechat_order);
    if ($result->return_code == 'SUCCESS' && $result->result_code == 'SUCCESS'){
      $code_url = $result->code_url;
    } else { // For any reason, we cannot get a preorder made by WeChat service
      $form['commerce_message'] = [
        '#markup' => '<div class="checkout-help">' . t('WeChat Pay QRcode is not avaiable at the moment. ' . $result->err_code_des),
        '#weight' => -10,
      ];

      return $form;
    }

    $barcode = new Barcode();
    // generate a barcode
    $bobj = $barcode->getBarcodeObj(
      'QRCODE,H',                     // barcode type and additional comma-separated parameters
      $code_url,          // data string to encode
      -4,                             // bar height (use absolute or negative value as multiplication factor)
      -4,                             // bar width (use absolute or negative value as multiplication factor)
      'black',                        // foreground color
      array(-2, -2, -2, -2)           // padding (use absolute or negative values as multiplication factors)
    )->setBackgroundColor('white'); // background color

    $form['commerce_message'] = [
      '#markup' => '<div class="checkout-help">' . t('Please scan the QR-Code below to complete the payment on your mobile WeChat App.') ,
      '#weight' => -10,
    ];

    $form['qrcode'] = [
      '#markup' => Markup::create($bobj->getHtmlDiv()),
    ];

    return $form;
  }

}
